{{
File  :  RGB_LED_Matrix_Demo.spin
Author:  John Zhong
Company: SimplyTronics
Version: 1.0
Date   : 2012-11-09

09/17/2016 - Francis Bauer
  Modified for 3x3 module matrix display of a 24x24 pixel Christmas Tree.     

}}

CON             
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

_pinTx   = 0                                       'RGB Serial Transmit Pin.
_pinRx   = 1                                       'RGB Serial Receive  Pin.

NumXrows = 24                                      'Number of X rows    of image.
NumYcols = 24                                      'Number of Y columns of image.

debug    = 0                                       'Serial Term debug mode 1 = on, 0 = off.
  
                  
OBJ
                                    
  RGB    : "RGB_LED_Matrix_Driver_Simple3.spin"
  Rand   : "RealRandom.spin"
  pst    : "Parallax Serial Terminal" 

PUB main | WaitTime

    if debug
      pst.Start(115_200)   

    Rand.start                                     'Start Random Number generator.
    RGB.Start(_pinRx,_pinTx, 19200)
    wait(2000)                                     'Pause to wait for the RGB Module start.            

    RGB.ChangeBaudrate(115200)                     'Increase baudrate.

    RGB.ClearScreen($0000_0000)                    'Send Clear Screen command. (AABB_GGRR)
    RGB.UpdateScreen                               'Update Screen for current command.

    DisplayFB                                      'Display FB
    DisplayText(string("09/30/2016"),0,11,$FF_00_00) 'Display Text (scrolling).

    RGB.ClearScreen($0000_0000)                    'Send Clear Screen command. (AABB_GGRR)
    RGB.UpdateScreen                               'Update Screen for current command. 
    DrawFrame(@XmasTree)
    RGB.UpdateScreen                               'Update Screen with current image/data.
    RGB.CopyGDRAM2(0)                              'Store current image for retrieval later. 
    WaitTime := 100
    wait(10 * WaitTime)
    repeat
      repeat 50
        DrawRandom                                 'Draw random lights on tree.
      DisplayText(string("   MERRY CHRISTMAS"),0,9,$00_00_FF) 'Display Text (scrolling).
      RGB.DisplayGDRAM(0)                          'Re-display previously stored image.  


PRI DisplayFB
{{ ********** Display "F" "B" ********** }}
  
    RGB.SetMode(RGB#_ModeNormal)
    '                          Foreground Color
    '                           B  G  R
    RGB.ASCII_0508NBG("F",0, 0,$FF_00_00 )
    RGB.SetMode(RGB#_ModeOverlay)
    '                          Foreground Color
    '                           B  G  R
    RGB.ASCII_0508NBG("B",3, 3,$00_00_FF  )
    RGB.UpdateScreen 
    RGB.SetMode(RGB#_ModeNormal) 
    

PRI DisplayText(DispText,TextXmin,TextY,TextColor) | prev_Xpos, Xpos
{/*******************************************************
              Display Text String 0508
*******************************************************/}
       prev_Xpos := 0   
       repeat Xpos from TextXmin to (-8 * strsize(DispText))    'Loop to potentially scroll text along X axis.
          String_0508NBG(DispText, prev_Xpos, TextY, $00000000) 'Blank previous text/string.   
          String_0508NBG(DispText, Xpos, TextY, TextColor)      'Write new text/string.
          RGB.updateScreen                         
          wait(75)
          prev_Xpos := Xpos

                
PRI String_0508NBG(stringPtr,StartX,StartY,ForegroundColor) | index    
{/*******************************************************
         Function: display the 0508 ASCII string
*******************************************************/}
    index := 0
    RGB.TxDatPackHead(RGB#_SetForegroundColor, ForegroundColor)
    RGB.TxDatPackHead(RGB#_SetPointerY, StartY)
      
    repeat strsize(stringPtr)
      RGB.TxDatPackHead(RGB#_SetPointerX, StartX + (index * 8))       
      RGB.TxDatPackHead(RGB#_DisplayChar0508NBG, byte[stringPtr + index])
      index++

          
PRI DrawFrame(Frame) | Xmin, Xmax, Xrow, Ycol, Color, Index
{{ ********** Display image ********** }}
    
    Index := 0                                     'Reset Index pointer.
    Ycol := byte[Frame][Index]                     'Obtain Ycol value.
    repeat while Ycol < NumYcols                   'Loop until End-of-dataset.
      Xmin := byte[Frame][++Index]
      Xmax := byte[Frame][++Index]
      Color.byte[3] := 0                           'Clear 1st byte in long. (High)
      Color.byte[2] := byte[Frame][++Index]        'Get Blue  channel value.
      Color.byte[1] := byte[Frame][++Index]        'Get Green channel value.
      Color.byte[0] := byte[Frame][++Index]        'Get Red   channel value. (Low)       
      repeat Xrow from Xmin to Xmax
        if debug
          pst.Str(string("Xrow: "))
          pst.Dec(Xrow)
          pst.Str(string(" Ycol: "))
          pst.Dec(Ycol)
          pst.Str(string(" Color: "))
          pst.Hex(Color,8)
          pst.Chars(pst#NL, 1)
        RGB.Dot(Xrow, Ycol, Color)                 'Send pixel color data to X,Y display position.
      Ycol := byte[Frame][++Index]                 'Obtain next Ycol value.
      if debug
        pst.Str(string("Next Ycol: "))
        pst.Dec(Ycol)
        pst.Chars(pst#NL, 1)    


PRI DrawRandom | index, bit, fill        
    repeat index from 0 to (NumYcols - 1)
      fill := long[@lights][index]                 'Obtain Col fill value.
      repeat bit from 0 to (NumXrows - 1)
        if get_bit(fill, bit)                      'Check to see if current position should be filled.
          RGB.Dot(bit, index, Rand.random)         'Fill current position with random colored dot.
    RGB.UpdateScreen


PUB get_bit(value, pos)                            'Return specified bit from value.
    return (value >> pos) & 1

            
PUB wait(Time_in_ms)                               'This routine has the program wait the specified number of milliseconds.
  waitcnt(((clkfreq / 1_000 * Time_in_ms - 3932) #> 381) + cnt)
  {{ This will delay/wait the specified number of milliseconds. This is SPIN code, and SPIN is interpreted,
     so it takes significantly longer to interpret and execute the waitcnt command than it does with PASM.
     The 3932 is to compensate for the time that it takes the interpreter to process the computation portion of the command.
     The #>381 insures that you never do a waitcnt with a value less than 381 more than the current CNT, because if you do
     that the counter will have already counted past your target value before the timing starts, and that means you will
     hang there while the counter loops through it's entire 32-bit range, somewhere around a minute (53 seconds).
     Waitcnt waits for the system counter to be exactly equal to the target value. }}

     
DAT

'In order to save memory, I converted the 576 longs that were used to describe the 24x24 BGR pixels of the Christmas Tree
'image into the XmasTree data structure.

'The image has been rotated 90 deg to display in the correct orientation with the ASCII Text screens.  So Col is actually
'specifying the vertical position and Row is specifying the horizontal position.  Confusing I know, but that is what
'was needed.

         '             Rows
         '33222222222211111111110000000000
Lights   '10987654321098765432109876543210
    long %00000000000000000000000000000000  'Col 00  Matrix defining where to do a random light fill.
    long %00000000000000000001100000000000  'Col 01
    long %00000000000000000011110000000000  'Col 02
    long %00000000000000000111111000000000  'Col 03
    long %00000000000000001111111100000000  'Col 04
    long %00000000000000011111111110000000  'Col 05
    long %00000000000000111111111111000000  'Col 06
    long %00000000000001111111111111100000  'Col 07
    long %00000000000000001111111100000000  'Col 08
    long %00000000000000001111111100000000  'Col 09
    long %00000000000000011111111110000000  'Col 10
    long %00000000000000111111111111000000  'Col 11
    long %00000000000001111111111111100000  'Col 12
    long %00000000000011111111111111110000  'Col 13
    long %00000000000111111111111111111000  'Col 14
    long %00000000000000011111111110000000  'Col 15
    long %00000000000000011111111110000000  'Col 16
    long %00000000000000111111111111000000  'Col 17
    long %00000000000001111111111111100000  'Col 18
    long %00000000000011111111111111110000  'Col 19
    long %00000000000111111111111111111000  'Col 20
    long %00000000001111111111111111111100  'Col 21
    long %00000000000000000000000000000000  'Col 22
    long %00000000000000000000000000000000  'Col 23

XmasTree 'Col,  RowStart, RowStop, Color(   B,   G,   R )
    byte    0,     11,       12,          $00, $FF, $00 'Green Christmas Tree.
    byte    1,     10,       13,          $00, $FF, $00    
    byte    2,     09,       14,          $00, $FF, $00    
    byte    3,     08,       15,          $00, $FF, $00    
    byte    4,     07,       16,          $00, $FF, $00    
    byte    5,     06,       17,          $00, $FF, $00
    byte    6,     05,       18,          $00, $FF, $00
    byte    7,     04,       19,          $00, $FF, $00
    byte    8,     03,       20,          $00, $FF, $00
    byte    9,     07,       16,          $00, $FF, $00
    byte   10,     06,       17,          $00, $FF, $00
    byte   11,     05,       18,          $00, $FF, $00
    byte   12,     04,       19,          $00, $FF, $00
    byte   13,     03,       20,          $00, $FF, $00
    byte   14,     02,       21,          $00, $FF, $00
    byte   15,     01,       22,          $00, $FF, $00
    byte   16,     06,       17,          $00, $FF, $00
    byte   17,     05,       18,          $00, $FF, $00
    byte   18,     04,       19,          $00, $FF, $00
    byte   19,     03,       20,          $00, $FF, $00
    byte   20,     02,       21,          $00, $FF, $00
    byte   21,     01,       22,          $00, $FF, $00
    byte   22,     00,       23,          $00, $FF, $00
    byte   23,     10,       13,          $13, $45, $8B    
    byte   24,      0,        0,          $00, $00, $00 'End-of-dataset     